﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Tetris
{
    // Block
    class Block
    {
        // position 
        Vector2 pos; // position in field(x: 0-9, y:-19): y can be less than 0
        int w = 25;
        int h = 25;

        // color
        Color color;

        // flag
        bool isDeleted;
        public bool IsDeleted
        {
            get
            {
                return this.isDeleted;
            }
        }

        public int X
        {
            set
            {
                this.pos.X = value;
            }
            get
            {
                return (int)this.pos.X;
            }
        }

        public int Y
        {
            set
            {
                this.pos.Y = value;
            }
            get
            {
                return (int)this.pos.Y;
            }
        }

        // If called, the block will be deleted
        public void MarkForDelete()
        {
            this.color = Color.White;
            this.isDeleted = true;
        }

        public void Draw(SpriteBatch spriteBatch, Texture2D texture, int offset_x, int offset_y){
            if (0 <= pos.X && pos.X < 20 && 0 <= pos.Y && pos.Y < 20)
            {
                spriteBatch.Draw(texture, new Rectangle(offset_x + w * X, offset_y + h * Y, w, h), new Color(color.R, color.G, color.B, 128));
                spriteBatch.Draw(texture, new Rectangle(offset_x + w * X + 1, offset_y + h * Y + 1, w - 2, h - 2), new Color(color.R, color.G, color.B, 192));
            }
        }

        public void Draw(SpriteBatch spriteBatch, Texture2D texture, int offset_x, int offset_y, int base_x, int base_y)
        {
            int dx = X - base_x; // relative position
            int dy = Y - base_y; // relative position
            spriteBatch.Draw(texture, new Rectangle(offset_x + w * dx, offset_y + h * dy, w, h), new Color(color.R, color.G, color.B, 128));
            spriteBatch.Draw(texture, new Rectangle(offset_x + w * dx + 1, offset_y + h * dy + 1, w - 2, h - 2), new Color(color.R, color.G, color.B, 192));
        }

        public Block(int x, int y, Color color)
        {
            this.pos = new Vector2(x, y);
            this.color = color;
        }
        public Block() : this(0, 0, Color.White) { }
        public Block(int x, int y) : this(x, y, Color.White){ }

    }
}
