﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Tetris
{
    class GameOverException : Exception { }

    class Field
    {
        Block[,] blocks;

        public Field()
        {
            blocks = new Block[10, 20];
        }

        // Add a block to blocks
        public void AddBlock(Block block)
        {
            try
            {
                blocks[block.X, block.Y] = block;
            }
            catch (IndexOutOfRangeException)
            {
                throw new GameOverException();
            }
        }

        // Scan blocks, and if blocks fill the line, mark it and return num of deleting line.
        public int MarkLinesForDelete()
        {
            bool filled;
            int numFilled = 0;
            for (int j = 0; j < 20; j++)
            {
                filled = false;
                for (int i = 0; i < 10; i++)
                {
                    if (blocks[i,j] == null)
                        break;
                    if (i == 9)
                        filled = true;
                }
                if (filled)
                {
                    numFilled++;
                    for (int i = 0; i < 10; i++)
                    {
                        blocks[i, j].MarkForDelete();
                    }
                }
            }
            return numFilled;
        }

        // Delete marked blocks and slide lines
        public void DeleteMarkedBlocks()
        {
            Block[,] new_blocks = new Block[10,20];
            int jj = 19;
            // scan from bottom
            for (int j = 19; j >= 0; j--)
            {
                if (blocks[0, j] != null && blocks[0, j].IsDeleted)
                    continue;
                for (int i = 0; i < 10; i++)
                {
                    if (blocks[i, j] != null)
                    {
                        new_blocks[i, jj] = blocks[i, j];
                        new_blocks[i, jj].X = i;
                        new_blocks[i, jj].Y = jj;
                    }
                }
                jj--;
            }
            blocks = new_blocks;
        }

        // If the field has block on (x, y), return true
        public bool HasBlock(int x, int y)
        {
            if (x >= 0 && x < 10 && y >= 0 && y < 20)
                if (blocks[x,y] != null)
                    return true;
            return false;
        }

        // Draw blocks
        public void DrawBlocks(SpriteBatch spriteBatch, Texture2D texture, int offset_x, int offset_y)
        {
            foreach (Block block in blocks)
            {
                if (block != null)
                    block.Draw(spriteBatch, texture, offset_x, offset_y);
            }
        }

    }
}
